#!/bin/sh
# This script must be able to run on busybox-powered devices, so uses POSIX shell instead of bash.

set -e #< Exit on error.

CUSTOMIZATION="networkoptix"
DISTRIB="nxgo-server-5.1.5.39242-linux_arm64.hanwha_edge1"

INSTALL_DIR="/opt/${CUSTOMIZATION}"
MODULE_DIR="${INSTALL_DIR}/mediaserver"
SERVER_PLUGINS_DIR="${MODULE_DIR}/bin/plugins"
BUNDLED_PLUGINS_FILE="${MODULE_DIR}/bundled_plugins.txt"
TAR_FILE="./${DISTRIB}.tar.gz"
BACKUP_PLUGINS_DIR=""

CONTROL_SCRIPT="${INSTALL_DIR}/mediaserver/bin/mediaserver.sh"

remove_bundled_plugins()
{
    if [ ! -d "${SERVER_PLUGINS_DIR}" ]; then
        return
    fi

    if [ ! -f "${BUNDLED_PLUGINS_FILE}" ]; then
        # Only the old versions of the Server (up to 5.1.4) do not generate the bundled_plugins.txt
        # file, so we create it for them right now. Remove this when the support for these versions
        # is dropped.
        cat <<EOF > "${BUNDLED_PLUGINS_FILE}"
libgeneric_multicast_plugin.so
libgeneric_rtsp_plugin.so
libhanwha_analytics_plugin.so
libmjpeg_link_plugin.so
libusb_camera_plugin.so
EOF
    fi

    xargs -I{} rm -rf "${SERVER_PLUGINS_DIR}/{}" < "${BUNDLED_PLUGINS_FILE}"
}

back_up_necessary_files()
{
    # Backup 3rd-party plugins.
    if [ -n "$(ls -A "${SERVER_PLUGINS_DIR}")" ]; then
        BACKUP_PLUGINS_DIR="$(mktemp -d)"
        cp -r "${SERVER_PLUGINS_DIR}/"* "${BACKUP_PLUGINS_DIR}/"
    fi
}

restore_backup_files()
{
    if [ -d "${BACKUP_PLUGINS_DIR}" ]; then
        # Restore plugin only if there is no such plugin in the target directory (i.e. no such
        # plugin is bundled with the new version), thus "-n" flag for "mv" command.
        mv -n "${BACKUP_PLUGINS_DIR}"/* "${SERVER_PLUGINS_DIR}"
        rm -Rf "${BACKUP_PLUGINS_DIR}"
    fi
}

create_bundled_plugins_list()
{
    echo "# List of plugins supplied with the Server package" > "${BUNDLED_PLUGINS_FILE}"
    ( #< Use subscript to avoid changing the current directory.
        cd "${SERVER_PLUGINS_DIR}"
        for plugin in *; do
            echo "${plugin}" >> "${BUNDLED_PLUGINS_FILE}"
        done
    )
}

upgrade_vms()
{
    if [ -f "${CONTROL_SCRIPT}" ]; then
        if "${CONTROL_SCRIPT}" status >/dev/null 2>&1; then
            was_running="1"
            "${CONTROL_SCRIPT}" stop
        else
            was_running="0"
        fi
    else
        was_running="0"
    fi

    remove_bundled_plugins
    back_up_necessary_files

    rm -rf "${MODULE_DIR}/lib"
    rm -rf "${MODULE_DIR}/bin/plugins"
    rm -rf "${MODULE_DIR}/bin/plugins_optional"
    mkdir -p "${INSTALL_DIR}"

    tar xfv "${TAR_FILE}" --dereference --directory "/"

    create_bundled_plugins_list

    mkdir -p "${INSTALL_DIR}/mediaserver/etc"

    restore_backup_files

    "${INSTALL_DIR}/mediaserver/bin/upgrade_config.sh" \
        "${INSTALL_DIR}/mediaserver/etc/mediaserver.conf" \
        "${INSTALL_DIR}/mediaserver/share/configs/mediaserver.conf" \
        "${INSTALL_DIR}/mediaserver/etc/mediaserver_conf_added_keys.txt" \
        "${INSTALL_DIR}/mediaserver/bin/config_helper.sh"

    if [ "${was_running}" = "1" ]; then
        "${CONTROL_SCRIPT}" start
    fi
}

main()
{
    if [ ! -z "$1" ]; then
        upgrade_vms >>"$1" 2>&1
    else
        upgrade_vms 2>&1
    fi
}

main "$@"
